/**
 * \file main.c
 *
 * \brief HTU21 Temperature & Humidity monitoring application file
 *
 * Copyright (c) 2014 Measurement Specialties. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 *
 * \asf_license_stop
 *
 */

/**
 * \mainpage HTU21 driver Application documentation
 *
 * \par Description
 *
 * This is a basic application for monitoring HTU21 temperature & humidity
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to system_init() and htu21_init()
 * -# Check presence of device : htu21_is_connected()
 * -# Reset htu21 : htu21_reset()
 * -# Read htu21 serial number : htu21_read_serial_number()
 * -# Set temperature and humidity resolution : htu21_set_resolution
 * -# While loop that monitors temperature and humidity every second : htu21_read_temperature_and_relative_humidity()
 *      - On each loop change the mode of access by calling htu21_set_i2c_master_mode()
 *      - During the loop, enable the heater for 10s using htu21_enable_heater() and come back to heater disabled with htu21_disable_heater()
 * 		- Retrieve the heater status  : htu21_get_heater_status()
 * 		- Compute compensated humidity : htu21_compute_compensated_humidity()
 * 		- Compute dew point : htu21_compute_dew_point()
 *      - When temperature increases, the LED0 will be switched on
 *      - When temperature decreases, the LED0 will be switched off
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
#include <asf.h>

uint64_t serial;
float temperature;
float relative_humidity;
float compensated_humidity;
float dew_point;
enum htu21_heater_status heater;

int main (void)
{
	enum htu21_status status;
	float last_temperature = 0;
	float variation = 0;
	uint8_t n=0;
	
	system_init();
	delay_init();

	// Configure device and enable
	htu21_init();
	
	if( !htu21_is_connected() )
		return -1;
	
	// Reset HTU21
	status = htu21_reset();
	// Current HW does not acknowledge the data on I2C reset transfer
	if( status != htu21_status_ok)
		return -1;
	
	// Read serial number
	status = htu21_read_serial_number(&serial);
	if( status != htu21_status_ok)
		return -1;

	// Configure resolution
	status = htu21_set_resolution(htu21_resolution_t_12b_rh_8b);
	if( status != htu21_status_ok)
		return -1;

	// Monitor temperature every 500ms
	while (1) {
		
		// Enable heater for 10s
		if( (n==10) ) {
			status = htu21_enable_heater();
			if( status != htu21_status_ok)
				return -1;
		}
		// Disable heater after 20s
		if( (n==20) ) {
			status = htu21_disable_heater();
			if( status != htu21_status_ok)
				return -1;
		}
		
		// Check heater status
		status = htu21_get_heater_status(&heater);
		if( status != htu21_status_ok)
			return -1;
		
		// Alternate between w and w/o hold temperature read
		if( n&1 )
			htu21_set_i2c_master_mode(htu21_i2c_no_hold);
		else
			htu21_set_i2c_master_mode(htu21_i2c_hold);
			
		status = htu21_read_temperature_and_relative_humidity(&temperature, &relative_humidity);
		if( status != htu21_status_ok)
			return -1;

		compensated_humidity = htu21_compute_compensated_humidity(temperature,relative_humidity);
		dew_point = htu21_compute_dew_point(temperature,relative_humidity);
		
		variation += temperature - last_temperature;

		// Look for significant temperature variation
		if (  variation >= 0.5 ) {
			// Yes, so turn LED on.
			port_pin_set_output_level(LED_0_PIN, LED_0_ACTIVE);
			variation = 0;
			} else if ( variation <= -0.5 ) {
			// No, so turn LED off.
			port_pin_set_output_level(LED_0_PIN, LED_0_INACTIVE);
			variation = 0;
		}
		
		delay_ms(1000);
		last_temperature = temperature;
		n++;
		if(n==50) n=0;
	}
	
	return 0;
}
